/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.api.generator.openapi;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.node.ObjectNode;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.apicurio.datamodels.openapi.models.OasDocument;
import io.syndesis.common.model.Violation;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link OpenApiModelInfo}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new OpenApiModelInfo.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOpenApiModelInfo.of()}.
 */
@Generated(from = "OpenApiModelInfo", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableOpenApiModelInfo
    implements OpenApiModelInfo {
  private final List<Violation> errors;
  private final OasDocument model;
  private final String resolvedSpecification;
  private final List<Violation> warnings;

  private ImmutableOpenApiModelInfo(
      Iterable<? extends Violation> errors,
      OasDocument model,
      String resolvedSpecification,
      Iterable<? extends Violation> warnings) {
    this.errors = createUnmodifiableList(false, createSafeList(errors, true, false));
    this.model = model;
    this.resolvedSpecification = resolvedSpecification;
    this.warnings = createUnmodifiableList(false, createSafeList(warnings, true, false));
    this.initShim = null;
  }

  private ImmutableOpenApiModelInfo(ImmutableOpenApiModelInfo.Builder builder) {
    this.model = builder.model;
    this.resolvedSpecification = builder.resolvedSpecification;
    if (builder.errorsIsSet()) {
      initShim.errors(createUnmodifiableList(true, builder.errors));
    }
    if (builder.warningsIsSet()) {
      initShim.warnings(createUnmodifiableList(true, builder.warnings));
    }
    this.errors = initShim.getErrors();
    this.warnings = initShim.getWarnings();
    this.initShim = null;
  }

  private ImmutableOpenApiModelInfo(
      ImmutableOpenApiModelInfo original,
      List<Violation> errors,
      OasDocument model,
      String resolvedSpecification,
      List<Violation> warnings) {
    this.errors = errors;
    this.model = model;
    this.resolvedSpecification = resolvedSpecification;
    this.warnings = warnings;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "OpenApiModelInfo", generator = "Immutables")
  private final class InitShim {
    private byte errorsBuildStage = STAGE_UNINITIALIZED;
    private List<Violation> errors;

    List<Violation> getErrors() {
      if (errorsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (errorsBuildStage == STAGE_UNINITIALIZED) {
        errorsBuildStage = STAGE_INITIALIZING;
        this.errors = createUnmodifiableList(false, createSafeList(getErrorsInitialize(), true, false));
        errorsBuildStage = STAGE_INITIALIZED;
      }
      return this.errors;
    }

    void errors(List<Violation> errors) {
      this.errors = errors;
      errorsBuildStage = STAGE_INITIALIZED;
    }

    private byte warningsBuildStage = STAGE_UNINITIALIZED;
    private List<Violation> warnings;

    List<Violation> getWarnings() {
      if (warningsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (warningsBuildStage == STAGE_UNINITIALIZED) {
        warningsBuildStage = STAGE_INITIALIZING;
        this.warnings = createUnmodifiableList(false, createSafeList(getWarningsInitialize(), true, false));
        warningsBuildStage = STAGE_INITIALIZED;
      }
      return this.warnings;
    }

    void warnings(List<Violation> warnings) {
      this.warnings = warnings;
      warningsBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (errorsBuildStage == STAGE_INITIALIZING) attributes.add("errors");
      if (warningsBuildStage == STAGE_INITIALIZING) attributes.add("warnings");
      return "Cannot build OpenApiModelInfo, attribute initializers form cycle " + attributes;
    }
  }

  private List<Violation> getErrorsInitialize() {
    return OpenApiModelInfo.super.getErrors();
  }

  private List<Violation> getWarningsInitialize() {
    return OpenApiModelInfo.super.getWarnings();
  }

  /**
   * @return The value of the {@code errors} attribute
   */
  @JsonProperty("errors")
  @Override
  public List<Violation> getErrors() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getErrors()
        : this.errors;
  }

  /**
   * @return The value of the {@code model} attribute
   */
  @JsonProperty("model")
  @Override
  public OasDocument getModel() {
    return model;
  }

  /**
   * @return The value of the {@code resolvedSpecification} attribute
   */
  @JsonProperty("resolvedSpecification")
  @Override
  public String getResolvedSpecification() {
    return resolvedSpecification;
  }

  /**
   * @return The value of the {@code warnings} attribute
   */
  @JsonProperty("warnings")
  @Override
  public List<Violation> getWarnings() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getWarnings()
        : this.warnings;
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link OpenApiModelInfo#getErrors() errors}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOpenApiModelInfo withErrors(Violation... elements) {
    List<Violation> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableOpenApiModelInfo(this, newValue, this.model, this.resolvedSpecification, this.warnings));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link OpenApiModelInfo#getErrors() errors}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of errors elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOpenApiModelInfo withErrors(Iterable<? extends Violation> elements) {
    if (this.errors == elements) return this;
    List<Violation> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableOpenApiModelInfo(this, newValue, this.model, this.resolvedSpecification, this.warnings));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OpenApiModelInfo#getModel() model} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for model (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOpenApiModelInfo withModel(OasDocument value) {
    if (this.model == value) return this;
    return validate(new ImmutableOpenApiModelInfo(this, this.errors, value, this.resolvedSpecification, this.warnings));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OpenApiModelInfo#getResolvedSpecification() resolvedSpecification} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for resolvedSpecification (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOpenApiModelInfo withResolvedSpecification(String value) {
    if (Objects.equals(this.resolvedSpecification, value)) return this;
    return validate(new ImmutableOpenApiModelInfo(this, this.errors, this.model, value, this.warnings));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link OpenApiModelInfo#getWarnings() warnings}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOpenApiModelInfo withWarnings(Violation... elements) {
    List<Violation> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableOpenApiModelInfo(this, this.errors, this.model, this.resolvedSpecification, newValue));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link OpenApiModelInfo#getWarnings() warnings}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of warnings elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOpenApiModelInfo withWarnings(Iterable<? extends Violation> elements) {
    if (this.warnings == elements) return this;
    List<Violation> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableOpenApiModelInfo(this, this.errors, this.model, this.resolvedSpecification, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOpenApiModelInfo} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOpenApiModelInfo
        && equalTo((ImmutableOpenApiModelInfo) another);
  }

  private boolean equalTo(ImmutableOpenApiModelInfo another) {
    return errors.equals(another.errors)
        && Objects.equals(model, another.model)
        && Objects.equals(resolvedSpecification, another.resolvedSpecification)
        && warnings.equals(another.warnings);
  }

  /**
   * Computes a hash code from attributes: {@code errors}, {@code model}, {@code resolvedSpecification}, {@code warnings}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + errors.hashCode();
    h += (h << 5) + Objects.hashCode(model);
    h += (h << 5) + Objects.hashCode(resolvedSpecification);
    h += (h << 5) + warnings.hashCode();
    return h;
  }


  /**
   * Prints the immutable value {@code OpenApiModelInfo} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "OpenApiModelInfo{"
        + "errors=" + errors
        + ", model=" + model
        + ", resolvedSpecification=" + resolvedSpecification
        + ", warnings=" + warnings
        + "}";
  }

  private transient volatile long lazyInitBitmap;

  private static final long API_VERSION_LAZY_INIT_BIT = 0x1L;

  private transient OpenApiVersion apiVersion;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link OpenApiModelInfo#getApiVersion() apiVersion} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code apiVersion} attribute
   */
  @Override
  public OpenApiVersion getApiVersion() {
    if ((lazyInitBitmap & API_VERSION_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & API_VERSION_LAZY_INIT_BIT) == 0) {
          this.apiVersion = Objects.requireNonNull(OpenApiModelInfo.super.getApiVersion(), "apiVersion");
          lazyInitBitmap |= API_VERSION_LAZY_INIT_BIT;
        }
      }
    }
    return apiVersion;
  }

  private static final long RESOLVED_JSON_GRAPH_LAZY_INIT_BIT = 0x2L;

  private transient ObjectNode resolvedJsonGraph;

  /**
   * {@inheritDoc}
   * <p>
   * Returns a lazily initialized value of the {@link OpenApiModelInfo#getResolvedJsonGraph() resolvedJsonGraph} attribute.
   * Initialized once and only once and stored for subsequent access with proper synchronization.
   * In case of any exception or error thrown by the lazy value initializer,
   * the result will not be memoised (i.e. remembered) and on next call computation
   * will be attempted again.
   * @return A lazily initialized value of the {@code resolvedJsonGraph} attribute
   */
  @Override
  public ObjectNode getResolvedJsonGraph() {
    if ((lazyInitBitmap & RESOLVED_JSON_GRAPH_LAZY_INIT_BIT) == 0) {
      synchronized (this) {
        if ((lazyInitBitmap & RESOLVED_JSON_GRAPH_LAZY_INIT_BIT) == 0) {
          this.resolvedJsonGraph = Objects.requireNonNull(OpenApiModelInfo.super.getResolvedJsonGraph(), "resolvedJsonGraph");
          lazyInitBitmap |= RESOLVED_JSON_GRAPH_LAZY_INIT_BIT;
        }
      }
    }
    return resolvedJsonGraph;
  }

  /**
   * Construct a new immutable {@code OpenApiModelInfo} instance.
   * @param errors The value for the {@code errors} attribute
   * @param model The value for the {@code model} attribute
   * @param resolvedSpecification The value for the {@code resolvedSpecification} attribute
   * @param warnings The value for the {@code warnings} attribute
   * @return An immutable OpenApiModelInfo instance
   */
  public static OpenApiModelInfo of(List<Violation> errors, OasDocument model, String resolvedSpecification, List<Violation> warnings) {
    return of((Iterable<? extends Violation>) errors, model, resolvedSpecification, (Iterable<? extends Violation>) warnings);
  }

  /**
   * Construct a new immutable {@code OpenApiModelInfo} instance.
   * @param errors The value for the {@code errors} attribute
   * @param model The value for the {@code model} attribute
   * @param resolvedSpecification The value for the {@code resolvedSpecification} attribute
   * @param warnings The value for the {@code warnings} attribute
   * @return An immutable OpenApiModelInfo instance
   */
  public static OpenApiModelInfo of(Iterable<? extends Violation> errors, OasDocument model, String resolvedSpecification, Iterable<? extends Violation> warnings) {
    return validate(new ImmutableOpenApiModelInfo(errors, model, resolvedSpecification, warnings));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOpenApiModelInfo validate(ImmutableOpenApiModelInfo instance) {
    Set<ConstraintViolation<ImmutableOpenApiModelInfo>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link OpenApiModelInfo} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable OpenApiModelInfo instance
   */
  public static OpenApiModelInfo copyOf(OpenApiModelInfo instance) {
    if (instance instanceof ImmutableOpenApiModelInfo) {
      return (ImmutableOpenApiModelInfo) instance;
    }
    return new OpenApiModelInfo.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link OpenApiModelInfo OpenApiModelInfo}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "OpenApiModelInfo", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long OPT_BIT_ERRORS = 0x1L;
    private static final long OPT_BIT_WARNINGS = 0x2L;
    private long optBits;

    private List<Violation> errors = new ArrayList<Violation>();
    private @Nullable OasDocument model;
    private @Nullable String resolvedSpecification;
    private List<Violation> warnings = new ArrayList<Violation>();

    /**
     * Creates a builder for {@link OpenApiModelInfo OpenApiModelInfo} instances.
     * <pre>
     * new OpenApiModelInfo.Builder()
     *    .addError|addAllErrors(io.syndesis.common.model.Violation) // {@link OpenApiModelInfo#getErrors() errors} elements
     *    .model(io.apicurio.datamodels.openapi.models.OasDocument | null) // nullable {@link OpenApiModelInfo#getModel() model}
     *    .resolvedSpecification(String | null) // nullable {@link OpenApiModelInfo#getResolvedSpecification() resolvedSpecification}
     *    .addWarning|addAllWarnings(io.syndesis.common.model.Violation) // {@link OpenApiModelInfo#getWarnings() warnings} elements
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof OpenApiModelInfo.Builder)) {
        throw new UnsupportedOperationException("Use: new OpenApiModelInfo.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code OpenApiModelInfo} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder createFrom(OpenApiModelInfo instance) {
      Objects.requireNonNull(instance, "instance");
      addAllErrors(instance.getErrors());
      OasDocument modelValue = instance.getModel();
      if (modelValue != null) {
        model(modelValue);
      }
      String resolvedSpecificationValue = instance.getResolvedSpecification();
      if (resolvedSpecificationValue != null) {
        resolvedSpecification(resolvedSpecificationValue);
      }
      addAllWarnings(instance.getWarnings());
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Adds one element to {@link OpenApiModelInfo#getErrors() errors} list.
     * @param element A errors element
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addError(Violation element) {
      this.errors.add(Objects.requireNonNull(element, "errors element"));
      optBits |= OPT_BIT_ERRORS;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Adds elements to {@link OpenApiModelInfo#getErrors() errors} list.
     * @param elements An array of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addErrors(Violation... elements) {
      for (Violation element : elements) {
        this.errors.add(Objects.requireNonNull(element, "errors element"));
      }
      optBits |= OPT_BIT_ERRORS;
      return (OpenApiModelInfo.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link OpenApiModelInfo#getErrors() errors} list.
     * @param elements An iterable of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("errors")
    public final OpenApiModelInfo.Builder errors(Iterable<? extends Violation> elements) {
      this.errors.clear();
      return addAllErrors(elements);
    }

    /**
     * Adds elements to {@link OpenApiModelInfo#getErrors() errors} list.
     * @param elements An iterable of errors elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addAllErrors(Iterable<? extends Violation> elements) {
      for (Violation element : elements) {
        this.errors.add(Objects.requireNonNull(element, "errors element"));
      }
      optBits |= OPT_BIT_ERRORS;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Initializes the value for the {@link OpenApiModelInfo#getModel() model} attribute.
     * @param model The value for model (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("model")
    public final OpenApiModelInfo.Builder model(OasDocument model) {
      this.model = model;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Initializes the value for the {@link OpenApiModelInfo#getResolvedSpecification() resolvedSpecification} attribute.
     * @param resolvedSpecification The value for resolvedSpecification (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("resolvedSpecification")
    public final OpenApiModelInfo.Builder resolvedSpecification(String resolvedSpecification) {
      this.resolvedSpecification = resolvedSpecification;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Adds one element to {@link OpenApiModelInfo#getWarnings() warnings} list.
     * @param element A warnings element
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addWarning(Violation element) {
      this.warnings.add(Objects.requireNonNull(element, "warnings element"));
      optBits |= OPT_BIT_WARNINGS;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Adds elements to {@link OpenApiModelInfo#getWarnings() warnings} list.
     * @param elements An array of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addWarnings(Violation... elements) {
      for (Violation element : elements) {
        this.warnings.add(Objects.requireNonNull(element, "warnings element"));
      }
      optBits |= OPT_BIT_WARNINGS;
      return (OpenApiModelInfo.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link OpenApiModelInfo#getWarnings() warnings} list.
     * @param elements An iterable of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("warnings")
    public final OpenApiModelInfo.Builder warnings(Iterable<? extends Violation> elements) {
      this.warnings.clear();
      return addAllWarnings(elements);
    }

    /**
     * Adds elements to {@link OpenApiModelInfo#getWarnings() warnings} list.
     * @param elements An iterable of warnings elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final OpenApiModelInfo.Builder addAllWarnings(Iterable<? extends Violation> elements) {
      for (Violation element : elements) {
        this.warnings.add(Objects.requireNonNull(element, "warnings element"));
      }
      optBits |= OPT_BIT_WARNINGS;
      return (OpenApiModelInfo.Builder) this;
    }

    /**
     * Builds a new {@link OpenApiModelInfo OpenApiModelInfo}.
     * @return An immutable instance of OpenApiModelInfo
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public OpenApiModelInfo build() {
      return ImmutableOpenApiModelInfo.validate(new ImmutableOpenApiModelInfo(this));
    }

    private boolean errorsIsSet() {
      return (optBits & OPT_BIT_ERRORS) != 0;
    }

    private boolean warningsIsSet() {
      return (optBits & OPT_BIT_WARNINGS) != 0;
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<>();
    } else {
      list = new ArrayList<>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }
}
